@@ -76,6 +76,7 @@ struct _GstID3Tag
 
   /* caps */
   GstID3ParseMode parse_mode;
+  GstCaps *found_caps;
 
   /* tags */
   GstTagList *event_tags;
@@ -123,7 +124,8 @@ GST_DEBUG_CATEGORY_EXTERN (mad_debug);
 static GstStaticPadTemplate id3_tag_src_any_template_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
-    GST_PAD_ALWAYS,
+  /* FIXME: for spider - should be GST_PAD_ALWAYS, */
+    GST_PAD_SOMETIMES,
     GST_STATIC_CAPS ("ANY")
     );
 
@@ -281,9 +283,28 @@ gst_id3_tag_class_init (gpointer g_class, gpointer class_data)
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_id3_tag_get_property);
 }
 
+static GstCaps *
+gst_id3_tag_get_caps (GstPad * pad)
+{
+  GstID3Tag *tag = GST_ID3_TAG (gst_pad_get_parent (pad));
+
+  if (tag->found_caps) {
+    GstCaps *caps = gst_caps_copy (tag->found_caps);
+
+    if (CAN_BE_MUXER (tag)) {
+      gst_caps_append (caps,
+          gst_caps_from_string ("application/x-gst-tags; application/x-id3"));
+    }
+    return caps;
+  } else {
+    return gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+  }
+}
+
 static void
 gst_id3_tag_add_src_pad (GstID3Tag * tag)
 {
+  g_assert (tag->srcpad == NULL);
   tag->srcpad =
       gst_pad_new_from_template (gst_element_class_get_pad_template
       (GST_ELEMENT_GET_CLASS (tag), "src"), "src");
@@ -295,6 +316,8 @@ gst_id3_tag_add_src_pad (GstID3Tag * tag)
       GST_DEBUG_FUNCPTR (gst_id3_tag_src_query));
   gst_pad_set_query_type_function (tag->srcpad,
       GST_DEBUG_FUNCPTR (gst_id3_tag_get_query_types));
+  gst_pad_set_getcaps_function (tag->srcpad,
+      GST_DEBUG_FUNCPTR (gst_id3_tag_get_caps));
   gst_pad_set_link_function (tag->srcpad,
       GST_DEBUG_FUNCPTR (gst_id3_tag_src_link));
   gst_element_add_pad (GST_ELEMENT (tag), tag->srcpad);
@@ -313,11 +336,13 @@ gst_id3_tag_init (GTypeInstance * instance, gpointer g_class)
     gst_element_add_pad (GST_ELEMENT (tag), tag->sinkpad);
     gst_pad_set_chain_function (tag->sinkpad,
         GST_DEBUG_FUNCPTR (gst_id3_tag_chain));
-
-    gst_id3_tag_add_src_pad (tag);
-
-    tag->parse_mode = GST_ID3_TAG_GET_CLASS (tag)->type;
   }
+  if (GST_ID3_TAG_GET_CLASS (tag)->type == GST_ID3_TAG_PARSE_MUX) {
+    /* only the muxer class here, all other use sometimes pads */
+    gst_id3_tag_add_src_pad (tag);
+  }
+  /* FIXME: for the alli^H^H^H^Hspider - gst_id3_tag_add_src_pad (tag); */
+  tag->parse_mode = GST_ID3_TAG_PARSE_BASE;
   tag->buffer = NULL;
 
   GST_FLAG_SET (tag, GST_ELEMENT_EVENT_AWARE);
@@ -497,164 +522,64 @@ gst_id3_tag_src_event (GstPad * pad, GstEvent * event)
   return FALSE;
 }
 
-static id3_utf8_t *
-mad_id3_parse_latin1_string (const id3_ucs4_t * ucs4)
-{
-  gsize bytes_read, size;
-  const gchar *env;
-  char *latin1, *ret = NULL;
-
-  latin1 = (char *) id3_ucs4_latin1duplicate (ucs4);
-  if (latin1 == NULL)
-    return NULL;
-
-  size = strlen (latin1);
-
-  env = g_getenv ("GST_ID3V2_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_ID3_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_TAG_ENCODING");
-
-  if (env && *env != '\0') {
-    gchar **c, **csets;
-
-    csets = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, -1);
-
-    for (c = csets; !ret && c && *c; ++c) {
-      gchar *utf8;
-
-      if ((utf8 =
-              g_convert (latin1, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
-        if (bytes_read == size) {
-          ret = strdup (utf8);
-        }
-        g_free (utf8);
-      }
-    }
-    g_strfreev (csets);
-  }
-
-  /* Try current locale (if not UTF-8). Should we really do this?
-   *  What if the tag is really correct and in ISO-8859-1 and the
-   *  current locale is some other charset where the full byte range
-   *  is valid? In those cases ISO-8859-1 would have to be put into
-   *  one of the above environment variables. Do the most common
-   *  non-Western and non-UTF8 character sets modify only the range
-   *  from 0x80-0xff, so that ASCII is still covered at least?) */
-  if (!ret && !g_get_charset (&env)) {
-    gchar *utf8;
-
-    if ((utf8 = g_locale_to_utf8 (latin1, size, &bytes_read, NULL, NULL))) {
-      if (bytes_read == size) {
-        ret = strdup (utf8);
-      }
-      g_free (utf8);
-    }
-  }
-
-  /* Try ISO-8859-1 (this conversion should always suceed) */
-  if (!ret) {
-    gchar *utf8;
-
-    utf8 =
-        g_convert (latin1, size, "UTF-8", "ISO-8859-1", &bytes_read, NULL,
-        NULL);
-    if (utf8 != NULL && bytes_read == size) {
-      ret = strdup (utf8);
-    }
-    g_free (utf8);
-  }
-
-  free (latin1);
-  return (id3_utf8_t *) ret;
-}
-
-static void
-mad_id3_parse_comment_frame (GstTagList * tlist, const struct id3_frame *frame)
-{
-  const id3_ucs4_t *ucs4;
-  gchar *utf8;
-
-  g_assert (frame->nfields >= 4);
-
-  ucs4 = id3_field_getfullstring (&frame->fields[3]);
-  g_assert (ucs4);
-
-  if (frame->fields[0].type == ID3_FIELD_TYPE_TEXTENCODING
-      && frame->fields[0].number.value == ID3_FIELD_TEXTENCODING_ISO_8859_1) {
-    utf8 = (gchar *) mad_id3_parse_latin1_string (ucs4);
-  } else {
-    utf8 = (gchar *) id3_ucs4_utf8duplicate (ucs4);
-  }
-
-  if (utf8 == NULL)
-    return;
-
-  if (!g_utf8_validate (utf8, -1, NULL)) {
-    g_warning ("converted string is not valid utf-8");
-    g_free (utf8);
-    return;
-  }
-
-  g_strchomp (utf8);
-
-  gst_tag_list_add (tlist, GST_TAG_MERGE_APPEND, GST_TAG_COMMENT, utf8, NULL);
-
-  g_free (utf8);
-}
-
 GstTagList *
 gst_mad_id3_to_tag_list (const struct id3_tag * tag)
 {
   const struct id3_frame *frame;
   const id3_ucs4_t *ucs4;
+  id3_utf8_t *utf8;
   GstTagList *tag_list;
-  gchar *utf8;
   guint i = 0;
 
   tag_list = gst_tag_list_new ();
 
   while ((frame = id3_tag_findframe (tag, NULL, i++)) != NULL) {
-    const union id3_field *field, *encfield;
+    const union id3_field *field;
     unsigned int nstrings, j;
     const gchar *tag_name;
 
-    tag_name = gst_tag_from_id3_tag (frame->id);
-    if (tag_name == NULL)
-      continue;
+    /* find me the function to query the frame id */
+    gchar *id = g_strndup (frame->id, 5);
 
-    if (strncmp (frame->id, "COMM", 5) == 0) {
-      mad_id3_parse_comment_frame (tag_list, frame);
+    tag_name = gst_tag_from_id3_tag (id);
+    if (tag_name == NULL) {
+      g_free (id);
       continue;
     }
 
-    if (frame->id[0] != 'T') {
-      g_warning ("don't know how to parse ID3v2 frame with ID '%s'", frame->id);
+    if (strcmp (id, "COMM") == 0) {
+      ucs4 = id3_field_getfullstring (&frame->fields[3]);
+      g_assert (ucs4);
+
+      utf8 = id3_ucs4_utf8duplicate (ucs4);
+      if (utf8 == 0)
+        continue;
+
+      if (!g_utf8_validate (utf8, -1, NULL)) {
+        g_warning ("converted string is not valid utf-8");
+        g_free (utf8);
+        continue;
+      }
+
+      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+          GST_TAG_COMMENT, utf8, NULL);
+
+      g_free (utf8);
       continue;
     }
 
-    g_assert (frame->nfields >= 2);
-
     field = &frame->fields[1];
     nstrings = id3_field_getnstrings (field);
-    encfield = &frame->fields[0];
 
     for (j = 0; j < nstrings; ++j) {
       ucs4 = id3_field_getstrings (field, j);
       g_assert (ucs4);
 
-      if (strncmp (frame->id, ID3_FRAME_GENRE, 5) == 0)
+      if (strcmp (id, ID3_FRAME_GENRE) == 0)
         ucs4 = id3_genre_name (ucs4);
 
-      if (encfield->type == ID3_FIELD_TYPE_TEXTENCODING
-          && encfield->number.value == ID3_FIELD_TEXTENCODING_ISO_8859_1) {
-        utf8 = (gchar *) mad_id3_parse_latin1_string (ucs4);
-      } else {
-        utf8 = (gchar *) id3_ucs4_utf8duplicate (ucs4);
-      }
-
-      if (utf8 == NULL)
+      utf8 = id3_ucs4_utf8duplicate (ucs4);
+      if (utf8 == 0)
         continue;
 
       if (!g_utf8_validate (utf8, -1, NULL)) {
@@ -729,13 +654,13 @@ gst_mad_id3_to_tag_list (const struct id3_tag * tag)
         }
         default:
           g_assert (gst_tag_get_type (tag_name) == G_TYPE_STRING);
-          g_strchomp (utf8);
           gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, utf8,
               NULL);
           break;
       }
       free (utf8);
     }
+    g_free (id);
   }
 
   return tag_list;
@@ -836,32 +761,11 @@ gst_mad_tag_list_to_id3_tag (GstTagList * list)
   gst_tag_list_foreach (list, tag_list_to_id3_tag_foreach, tag);
   return tag;
 }
-
-static void
-gst_id3_tag_dump_tag_list (const GstTagList * taglist, const gchar * msg)
-{
-  if (taglist && gst_structure_n_fields (taglist) > 0) {
-    gchar *s = gst_structure_to_string (taglist);
-
-    GST_DEBUG ("%s: %s", msg, s);
-    g_free (s);
-  } else {
-    GST_DEBUG ("%s: no tags", msg);
-  }
-}
-
 static GstTagList *
 gst_id3_tag_get_tag_to_render (GstID3Tag * tag)
 {
-  const GstTagList *set_tags = NULL;
   GstTagList *ret = NULL;
 
-  set_tags = gst_tag_setter_get_list (GST_TAG_SETTER (tag));
-
-  gst_id3_tag_dump_tag_list (set_tags, "Set tags");
-  gst_id3_tag_dump_tag_list (tag->event_tags, "Event tags");
-  gst_id3_tag_dump_tag_list (tag->parsed_tags, "Parsed tags");
-
   if (tag->event_tags)
     ret = gst_tag_list_copy (tag->event_tags);
   if (ret) {
@@ -870,16 +774,12 @@ gst_id3_tag_get_tag_to_render (GstID3Tag * tag)
   } else if (tag->parsed_tags) {
     ret = gst_tag_list_copy (tag->parsed_tags);
   }
-
-  if (ret && set_tags) {
-    gst_tag_list_insert (ret, set_tags,
+  if (ret && gst_tag_setter_get_list (GST_TAG_SETTER (tag))) {
+    gst_tag_list_insert (ret, gst_tag_setter_get_list (GST_TAG_SETTER (tag)),
         gst_tag_setter_get_merge_mode (GST_TAG_SETTER (tag)));
-  } else if (set_tags) {
-    ret = gst_tag_list_copy (set_tags);
+  } else if (gst_tag_setter_get_list (GST_TAG_SETTER (tag))) {
+    ret = gst_tag_list_copy (gst_tag_setter_get_list (GST_TAG_SETTER (tag)));
   }
-
-  gst_id3_tag_dump_tag_list (ret, "Resulting tags");
-
   return ret;
 }
 static void
@@ -891,7 +791,7 @@ gst_id3_tag_handle_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_DISCONTINUOUS:
       switch (tag->state) {
         case GST_ID3_TAG_STATE_READING_V2_TAG:{
-          gint64 value;
+          guint64 value;
 
           if (gst_event_discont_get_value (event, GST_FORMAT_BYTES, &value) ||
               gst_event_discont_get_value (event, GST_FORMAT_DEFAULT, &value)) {
@@ -995,6 +895,91 @@ gst_id3_tag_handle_event (GstPad * pad, GstEvent * event)
   }
   return;
 }
+typedef struct
+{
+  guint best_probability;
+  GstCaps *caps;
+  GstBuffer *buffer;
+}
+SimpleTypeFind;
+guint8 *
+simple_find_peek (gpointer data, gint64 offset, guint size)
+{
+  SimpleTypeFind *find = (SimpleTypeFind *) data;
+
+  if (offset < 0)
+    return NULL;
+
+  if (GST_BUFFER_SIZE (find->buffer) >= offset + size) {
+    return GST_BUFFER_DATA (find->buffer) + offset;
+  }
+  return NULL;
+}
+static void
+simple_find_suggest (gpointer data, guint probability, const GstCaps * caps)
+{
+  SimpleTypeFind *find = (SimpleTypeFind *) data;
+
+  if (probability > find->best_probability) {
+    gst_caps_replace (&find->caps, gst_caps_copy (caps));
+    find->best_probability = probability;
+  }
+}
+static GstCaps *
+gst_id3_tag_do_typefind (GstID3Tag * tag, GstBuffer * buffer)
+{
+  GList *walk, *type_list;
+  SimpleTypeFind find;
+  GstTypeFind gst_find;
+
+  /* this will help us detecting the media stream type after
+   * this id3 thingy... Please note that this is a cruel hack
+   * for as long as spider doesn't support multi-type-finding.
+   */
+  walk = type_list = gst_type_find_factory_get_list ();
+
+  find.buffer = buffer;
+  find.best_probability = 0;
+  find.caps = NULL;
+  gst_find.data = &find;
+  gst_find.peek = simple_find_peek;
+  gst_find.suggest = simple_find_suggest;
+  while (walk) {
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (walk->data);
+
+    gst_type_find_factory_call_function (factory, &gst_find);
+    if (find.best_probability >= GST_TYPE_FIND_MAXIMUM)
+      break;
+    walk = g_list_next (walk);
+  }
+  g_list_free (type_list);
+  if (find.best_probability > 0) {
+    return find.caps;
+  } else {
+    GST_ELEMENT_ERROR (tag, CORE, CAPS, (NULL), ("no caps found"));
+    return NULL;
+  }
+}
+static gboolean
+gst_id3_tag_do_caps_nego (GstID3Tag * tag, GstBuffer * buffer)
+{
+  if (buffer != NULL && CAN_BE_DEMUXER (tag)) {
+    tag->found_caps = gst_id3_tag_do_typefind (tag, buffer);
+    if (!tag->found_caps) {
+      return FALSE;
+    }
+  }
+  if (!tag->srcpad)
+    gst_id3_tag_add_src_pad (tag);
+  if (!gst_pad_is_linked (tag->srcpad)) {
+    GST_DEBUG_OBJECT (tag, "srcpad not linked, not proceeding");
+    tag->parse_mode = GST_ID3_TAG_GET_CLASS (tag)->type;
+    return TRUE;
+  } else {
+    GST_DEBUG_OBJECT (tag, "renegotiating");
+    return gst_pad_renegotiate (tag->srcpad) != GST_PAD_LINK_REFUSED;
+  }
+}
 
 static GstPadLinkReturn
 gst_id3_tag_src_link (GstPad * pad, const GstCaps * caps)
@@ -1004,6 +989,8 @@ gst_id3_tag_src_link (GstPad * pad, const GstCaps * caps)
 
   tag = GST_ID3_TAG (gst_pad_get_parent (pad));
 
+  if (!tag->found_caps && CAN_BE_DEMUXER (tag))
+    return GST_PAD_LINK_DELAYED;
   if (!CAN_BE_MUXER (tag) || !CAN_BE_DEMUXER (tag)) {
     tag->parse_mode = GST_ID3_TAG_GET_CLASS (tag)->type;
     return GST_PAD_LINK_OK;
@@ -1045,7 +1032,6 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
 {
   GstID3Tag *tag;
   GstBuffer *buffer;
-  GstTagList *newtag;
 
   /* handle events */
   if (GST_IS_EVENT (data)) {
@@ -1077,21 +1063,30 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
       if (GST_BUFFER_SIZE (tag->buffer) < 128)
         return;
       g_assert (tag->v1tag_size == 0);
-      newtag = gst_tag_list_new_from_id3v1 (GST_BUFFER_DATA (tag->buffer));
-      if (newtag) {
-        tag->v1tag_size = 128;
+      tag->v1tag_size = id3_tag_query (GST_BUFFER_DATA (tag->buffer),
+          GST_BUFFER_SIZE (tag->buffer));
+      if (tag->v1tag_size == 128) {
+        GstTagList *newtag;
+
+        newtag = gst_tag_list_new_from_id3v1 (GST_BUFFER_DATA (tag->buffer));
         GST_LOG_OBJECT (tag, "have read ID3v1 tag");
-        if (tag->parsed_tags) {
-          /* FIXME: use append/prepend here ? */
-          gst_tag_list_insert (tag->parsed_tags, newtag,
-              tag->prefer_v1tag ? GST_TAG_MERGE_REPLACE : GST_TAG_MERGE_KEEP);
-          gst_tag_list_free (newtag);
+        if (newtag) {
+          if (tag->parsed_tags) {
+            /* FIXME: use append/prepend here ? */
+            gst_tag_list_insert (tag->parsed_tags, newtag,
+                tag->prefer_v1tag ? GST_TAG_MERGE_REPLACE : GST_TAG_MERGE_KEEP);
+            gst_tag_list_free (newtag);
+          } else {
+            tag->parsed_tags = newtag;
+          }
         } else {
-          tag->parsed_tags = newtag;
+          GST_WARNING_OBJECT (tag, "detected ID3v1 tag, but couldn't parse it");
         }
+      } else if (tag->v1tag_size != 0) {
+        GST_WARNING_OBJECT (tag, "bad non-ID3v1 tag at end of file");
       } else {
-        GST_WARNING_OBJECT (tag, "no ID3v1 tag at end of file");
-        tag->v1tag_offset = G_MAXUINT64;
+        GST_LOG_OBJECT (tag, "no ID3v1 tag (%" G_GUINT64_FORMAT ")",
+            GST_BUFFER_OFFSET (tag->buffer));
       }
       gst_data_unref (GST_DATA (tag->buffer));
       tag->buffer = NULL;
@@ -1171,6 +1166,9 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
             GST_BUFFER_OFFSET_END (tag->buffer) + tag->v2tag_size;
       gst_data_unref (GST_DATA (tag->buffer));
       tag->buffer = NULL;
+      if (tag->found_caps == NULL)
+        if (!gst_id3_tag_do_caps_nego (tag, buffer))
+          return;
       /* seek to ID3v1 tag */
       if (gst_pad_send_event (GST_PAD_PEER (tag->sinkpad),
               gst_event_new_seek (GST_FORMAT_BYTES | GST_SEEK_METHOD_END |
@@ -1231,7 +1229,6 @@ gst_id3_tag_chain (GstPad * pad, GstData * data)
             GstBuffer *sub = gst_buffer_create_sub (buffer, 0,
                 buffer->size - 128);
 
-            GST_BUFFER_OFFSET (sub) = GST_BUFFER_OFFSET (buffer);
             gst_data_unref (GST_DATA (buffer));
             buffer = sub;
           }
@@ -1296,7 +1293,11 @@ gst_id3_tag_change_state (GstElement * element)
         gst_data_unref (GST_DATA (tag->buffer));
         tag->buffer = NULL;
       }
-      tag->parse_mode = GST_ID3_TAG_GET_CLASS (tag)->type;
+      if (tag->found_caps) {
+        gst_caps_free (tag->found_caps);
+        tag->found_caps = NULL;
+      }
+      tag->parse_mode = GST_ID3_TAG_PARSE_BASE;
       break;
     case GST_STATE_READY_TO_NULL:
       break;
@@ -1315,15 +1316,13 @@ plugin_init (GstPlugin * plugin)
 
   if (!gst_element_register (plugin, "mad", GST_RANK_PRIMARY,
           gst_mad_get_type ())
-      || !gst_element_register (plugin, "id3demux", GST_RANK_NONE,
+      || !gst_element_register (plugin, "id3demux", GST_RANK_PRIMARY,
           gst_id3_tag_get_type (GST_ID3_TAG_PARSE_DEMUX))
       || !gst_element_register (plugin, "id3mux", GST_RANK_NONE,        /* removed for spider */
           gst_id3_tag_get_type (GST_ID3_TAG_PARSE_MUX))
       /* FIXME 0.9: remove this element */
       || !gst_element_register (plugin, "id3tag", GST_RANK_NONE,
-          gst_id3_tag_get_type (GST_ID3_TAG_PARSE_ANY))
-      || !gst_element_register (plugin, "id3demuxbin", GST_RANK_PRIMARY,
-          gst_id3demux_bin_get_type ())) {
+          gst_id3_tag_get_type (GST_ID3_TAG_PARSE_ANY))) {
     return FALSE;
   }
 
