@@ -170,7 +170,6 @@ gst_speex_dec_init (GstSpeexDec * dec)
       (&speex_dec_sink_factory), "sink");
   gst_pad_set_chain_function (dec->sinkpad, speex_dec_chain);
   gst_pad_set_formats_function (dec->sinkpad, speex_dec_get_formats);
-  gst_pad_set_convert_function (dec->sinkpad, speex_dec_convert);
   gst_element_add_pad (GST_ELEMENT (dec), dec->sinkpad);
 
   dec->srcpad =
@@ -204,10 +203,6 @@ speex_dec_convert (GstPad * pad,
   if (dec->packetno < 1)
     return FALSE;
 
-  if (pad == dec->sinkpad &&
-      (src_format == GST_FORMAT_BYTES || *dest_format == GST_FORMAT_BYTES))
-    return FALSE;
-
   switch (src_format) {
     case GST_FORMAT_TIME:
       switch (*dest_format) {
@@ -287,7 +282,7 @@ speex_dec_src_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:{
-      gint64 value;
+      guint64 value;
       GstFormat my_format = GST_FORMAT_DEFAULT;
 
       /* convert to samples_out */
@@ -315,12 +310,13 @@ speex_dec_src_event (GstPad * pad, GstEvent * event)
 static void
 speex_dec_event (GstSpeexDec * dec, GstEvent * event)
 {
-  gint64 value, time, bytes;
+  guint64 value, time, bytes;
 
   GST_LOG_OBJECT (dec, "handling event");
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_DISCONTINUOUS:
-      if (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT, &value)) {
+      if (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT,
+              (gint64 *) & value)) {
         dec->samples_out = value;
         GST_DEBUG_OBJECT (dec,
             "setting samples_out to %" G_GUINT64_FORMAT " after discont",
@@ -385,8 +381,8 @@ speex_dec_chain (GstPad * pad, GstData * data)
     GstCaps *caps;
 
     /* get the header */
-    dec->header = speex_packet_to_header ((gchar *) GST_BUFFER_DATA (buf),
-        GST_BUFFER_SIZE (buf));
+    dec->header =
+        speex_packet_to_header (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
     gst_data_unref (data);
     if (!dec->header) {
       GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
@@ -443,9 +439,8 @@ speex_dec_chain (GstPad * pad, GstData * data)
     gchar *encoder = NULL;
 
     /* FIXME parse comments */
-    GstTagList *list =
-        gst_tag_list_from_vorbiscomment_buffer (buf, (guint8 *) "",
-        1, &encoder);
+    GstTagList *list = gst_tag_list_from_vorbiscomment_buffer (buf, "", 1,
+        &encoder);
 
     gst_data_unref (data);
 
@@ -458,8 +453,6 @@ speex_dec_chain (GstPad * pad, GstData * data)
           GST_TAG_ENCODER, encoder, NULL);
       g_free (encoder);
     }
-    gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-        GST_TAG_AUDIO_CODEC, "Speex", NULL);
     /*
        gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
        GST_TAG_ENCODER_VERSION, dec->vi.version, NULL);
@@ -479,7 +472,7 @@ speex_dec_chain (GstPad * pad, GstData * data)
     gint i;
 
     /* send data to the bitstream */
-    speex_bits_read_from (&dec->bits, (gchar *) GST_BUFFER_DATA (buf),
+    speex_bits_read_from (&dec->bits, GST_BUFFER_DATA (buf),
         GST_BUFFER_SIZE (buf));
     gst_data_unref (data);
 
