@@ -41,8 +41,8 @@ enum
 
 enum
 {
-  ARG_0,
-  /* FILL ME */
+  ARG_0
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate video_template_factory =
@@ -118,6 +118,7 @@ gst_swfdec_get_type (void)
       0,
       (GInstanceInitFunc) gst_swfdec_init,
     };
+
     swfdec_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstSwfdec", &swfdec_info, 0);
   }
@@ -532,6 +533,7 @@ gst_swfdec_src_event (GstPad * pad, GstEvent * event)
   gboolean res = TRUE;
   GstSwfdec *swfdec;
   static const GstFormat formats[] = { GST_FORMAT_TIME, GST_FORMAT_BYTES };
+
 #define MAX_SEEK_FORMATS 1      /* we can only do time seeking for now */
   gint i;
 
