@@ -253,6 +253,7 @@ gst_audio_convert_chain (GstPad * pad, GstData * data)
     gst_pad_event_default (pad, GST_EVENT (buf));
     return;
   }
+  g_return_if_fail (GST_IS_BUFFER (buf));
 
   if (!gst_pad_is_negotiated (this->sink)) {
     GST_ELEMENT_ERROR (this, CORE, NEGOTIATION, (NULL),
@@ -372,7 +373,8 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
 {
   GstAudioConvert *this;
   GstPad *otherpad;
-  GstAudioConvertCaps ac_caps, other_ac_caps;
+  GstAudioConvertCaps ac_caps = { 0 }, other_ac_caps = {
+  0};
   GstCaps *othercaps;
   guint i;
   GstPadLinkReturn ret;
@@ -384,10 +386,6 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
   this = GST_AUDIO_CONVERT (GST_OBJECT_PARENT (pad));
   otherpad = (pad == this->src ? this->sink : this->src);
 
-  /* negotiate sinkpad first */
-  if (pad == this->src && !gst_pad_is_negotiated (this->sink))
-    return GST_PAD_LINK_DELAYED;
-
   if (!gst_audio_convert_parse_caps (caps, &ac_caps))
     return GST_PAD_LINK_REFUSED;
 
@@ -402,7 +400,8 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
         gst_structure_set (structure, "buffer-frames", G_TYPE_INT,
             ac_caps.buffer_frames, NULL);
       } else {
-        gst_structure_set (structure, "buffer-frames", G_TYPE_INT, 0, NULL);
+        gst_structure_set (structure, "buffer-frames", GST_TYPE_INT_RANGE, 0,
+            G_MAXINT, NULL);
       }
     }
   }
@@ -463,6 +462,7 @@ _fixate_caps_to_int (GstCaps ** caps, const gchar * field, gint value)
     gst_caps_append (try, gst_caps_new_simple ("audio/x-raw-float", field,
             GST_TYPE_INT_RANGE, value, G_MAXINT, NULL));
     isect_higher = gst_caps_intersect (*caps, try);
+    gst_caps_free (try);
     /* FIXME: why choose to end up with the higher range, and not the fixed
      * value ? */
     if (!gst_caps_is_empty (isect_higher)) {
@@ -550,6 +550,8 @@ gst_audio_convert_get_buffer (GstBuffer * buf, guint size)
 {
   GstBuffer *ret;
 
+  g_assert (GST_IS_BUFFER (buf));
+
   GST_LOG
       ("new buffer of size %u requested. Current is: data: %p - size: %u - maxsize: %u",
       size, buf->data, buf->size, buf->maxsize);
