@@ -66,43 +66,42 @@ enum
       "signed = (boolean) true")
 #if 0
   /* disabled because it segfaults */
-#define NOTHING "audio/x-raw-float, " \
-    "rate = (int) [ 1, MAX ], " \
-    "channels = (int) [ 1, MAX ], " \
+"audio/x-raw-float, "
+    "rate = (int) [ 1, MAX ], "
+    "channels = (int) [ 1, MAX ], "
     "endianness = (int) BYTE_ORDER, " "width = (int) 32")
 #endif
-static GstStaticPadTemplate gst_audioscale_sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+     static GstStaticPadTemplate gst_audioscale_sink_template =
+         GST_STATIC_PAD_TEMPLATE ("sink",
+         GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
-static GstStaticPadTemplate gst_audioscale_src_template =
-GST_STATIC_PAD_TEMPLATE ("src",
-    GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+     static GstStaticPadTemplate gst_audioscale_src_template =
+         GST_STATIC_PAD_TEMPLATE ("src",
+         GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
 #define GST_TYPE_AUDIOSCALE_METHOD (gst_audioscale_method_get_type())
-static GType
-gst_audioscale_method_get_type (void)
-{
-  static GType audioscale_method_type = 0;
-  static GEnumValue audioscale_methods[] = {
-    {
-        GST_RESAMPLE_NEAREST, "0", "Nearest"}
-    ,
-    {
-        GST_RESAMPLE_BILINEAR, "1", "Bilinear"}
-    , {
-        GST_RESAMPLE_SINC, "2", "Sinc"}
-    , {
-        0, NULL, NULL}
-    ,
-  };
+     static GType gst_audioscale_method_get_type (void)
+     {
+       static GType audioscale_method_type = 0;
+       static GEnumValue audioscale_methods[] =
+       {
+         {
+         GST_RESAMPLE_NEAREST, "0", "Nearest"}
+         ,
+         {
+         GST_RESAMPLE_BILINEAR, "1", "Bilinear"}
+         , {
+         GST_RESAMPLE_SINC, "2", "Sinc"}
+         , {
+         0, NULL, NULL}
+       ,};
 
-  if (!audioscale_method_type) {
-    audioscale_method_type = g_enum_register_static ("GstAudioscaleMethod",
-        audioscale_methods);
-  }
-  return audioscale_method_type;
-}
+       if (!audioscale_method_type) {
+         audioscale_method_type = g_enum_register_static ("GstAudioscaleMethod",
+             audioscale_methods);
+       }
+       return audioscale_method_type;
+     }
 
 static void gst_audioscale_base_init (gpointer g_class);
 static void gst_audioscale_class_init (AudioscaleClass * klass);
@@ -110,7 +109,6 @@ static void gst_audioscale_init (Audioscale * audioscale);
 static void gst_audioscale_dispose (GObject * object);
 
 static void gst_audioscale_chain (GstPad * pad, GstData * _data);
-static GstElementStateReturn gst_audioscale_change_state (GstElement * element);
 
 static void gst_audioscale_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -123,31 +121,30 @@ static GstElementClass *parent_class = NULL;
 
 /*static guint gst_audioscale_signals[LAST_SIGNAL] = { 0 }; */
 
-GType
-audioscale_get_type (void)
+GType audioscale_get_type (void)
 {
   static GType audioscale_type = 0;
 
-  if (!audioscale_type) {
-    static const GTypeInfo audioscale_info = {
-      sizeof (AudioscaleClass),
-      gst_audioscale_base_init,
-      NULL,
-      (GClassInitFunc) gst_audioscale_class_init,
-      NULL,
-      NULL,
-      sizeof (Audioscale), 0, (GInstanceInitFunc) gst_audioscale_init,
-    };
+  if (!audioscale_type)
+  {
+    static const GTypeInfo audioscale_info =
+    {
+    sizeof (AudioscaleClass),
+          gst_audioscale_base_init,
+          NULL,
+          (GClassInitFunc) gst_audioscale_class_init,
+          NULL,
+          NULL,
+          sizeof (Audioscale), 0, (GInstanceInitFunc) gst_audioscale_init,};
 
     audioscale_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "GstAudioscale",
+        g_type_register_static (GST_TYPE_ELEMENT, "Audioscale",
         &audioscale_info, 0);
   }
   return audioscale_type;
 }
 
-static void
-gst_audioscale_base_init (gpointer g_class)
+static void gst_audioscale_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
@@ -159,8 +156,7 @@ gst_audioscale_base_init (gpointer g_class)
   gst_element_class_set_details (gstelement_class, &gst_audioscale_details);
 }
 
-static void
-gst_audioscale_class_init (AudioscaleClass * klass)
+static void gst_audioscale_class_init (AudioscaleClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
@@ -171,7 +167,6 @@ gst_audioscale_class_init (AudioscaleClass * klass)
   gobject_class->set_property = gst_audioscale_set_property;
   gobject_class->get_property = gst_audioscale_get_property;
   gobject_class->dispose = gst_audioscale_dispose;
-  gstelement_class->change_state = gst_audioscale_change_state;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FILTERLEN,
       g_param_spec_int ("filter_length", "filter_length", "filter_length",
@@ -188,55 +183,125 @@ gst_audioscale_class_init (AudioscaleClass * klass)
       "audioscale element");
 }
 
-static GstStaticCaps gst_audioscale_passthru_caps =
-GST_STATIC_CAPS ("audio/x-raw-int, channels = [ 3, MAX ]");
-static GstStaticCaps gst_audioscale_convert_caps =
-GST_STATIC_CAPS ("audio/x-raw-int, channels = [ 1, 2 ]");
-
-static GstCaps *
-gst_audioscale_expand_caps (const GstCaps * caps)
+static void gst_audioscale_expand_value (GValue * dest, const GValue * src)
 {
-  GstCaps *caps1, *caps2;
-  int i;
+  int rate_min, rate_max;
 
-  caps1 = gst_caps_intersect (caps,
-      gst_static_caps_get (&gst_audioscale_passthru_caps));
-  caps2 = gst_caps_intersect (caps,
-      gst_static_caps_get (&gst_audioscale_convert_caps));
+  if (G_VALUE_TYPE (src) == G_TYPE_INT ||
+      G_VALUE_TYPE (src) == GST_TYPE_INT_RANGE)
+  {
+    if (G_VALUE_TYPE (src) == G_TYPE_INT) {
+      rate_min = g_value_get_int (src);
+      rate_max = rate_min;
+    } else
+    {
+      rate_min = gst_value_get_int_range_min (src);
+      rate_max = gst_value_get_int_range_max (src);
+    }
 
-  for (i = 0; i < gst_caps_get_size (caps2); i++) {
-    GstStructure *structure = gst_caps_get_structure (caps2, i);
+    rate_min = 1;
+    rate_max = G_MAXINT;
 
-    gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-        NULL);
+    g_value_init (dest, GST_TYPE_INT_RANGE);
+    gst_value_set_int_range (dest, rate_min, rate_max);
+    return;
   }
 
-  gst_caps_append (caps1, caps2);
+  if (G_VALUE_TYPE (src) == GST_TYPE_LIST) {
+    int i;
 
-  return caps1;
+    g_value_init (dest, GST_TYPE_LIST);
+    for (i = 0; i < gst_value_list_get_size (src); i++) {
+      const GValue *s = gst_value_list_get_value (src, i);
+      GValue d =
+      {
+      0};
+      int j;
+
+      gst_audioscale_expand_value (&d, s);
+
+      for (j = 0; j < gst_value_list_get_size (dest); j++) {
+        const GValue *s2 = gst_value_list_get_value (dest, j);
+        GValue d2 =
+        {
+        0};
+
+        gst_value_union (&d2, &d, s2);
+        if (G_VALUE_TYPE (&d2) == GST_TYPE_INT_RANGE) {
+          g_value_unset ((GValue *) s2);
+          gst_value_init_and_copy ((GValue *) s2, &d2);
+          break;
+        }
+        g_value_unset (&d2);
+      }
+      if (j == gst_value_list_get_size (dest)) {
+        gst_value_list_append_value (dest, &d);
+      }
+      g_value_unset (&d);
+    }
+
+    if (gst_value_list_get_size (dest) == 1) {
+      const GValue *s = gst_value_list_get_value (dest, 0);
+      GValue d =
+      {
+      0};
+
+      gst_value_init_and_copy (&d, s);
+      g_value_unset (dest);
+      gst_value_init_and_copy (dest, &d);
+      g_value_unset (&d);
+    }
+
+    return;
+  }
+
+  GST_ERROR ("unexpected value type");
 }
 
-static GstCaps *
-gst_audioscale_getcaps (GstPad * pad)
+static void gst_audioscale_expand_caps (GstCaps * caps)
+{
+  gint i;
+
+  /* we do this hack, because the audioscale lib doesn't handle
+   * rate conversions larger than a factor of 2 */
+  /* UPDATE: allowed for n iterations so can handle any factor */
+  for (i = 0; i < gst_caps_get_size (caps); i++) {
+    GstStructure *structure = gst_caps_get_structure (caps, i);
+    const GValue *value;
+    GValue dest =
+    {
+    0};
+
+    value = gst_structure_get_value (structure, "rate");
+    if (value == NULL) {
+      GST_ERROR ("caps structure doesn't have required rate field");
+      return;
+    }
+
+    gst_audioscale_expand_value (&dest, value);
+
+    gst_structure_set_value (structure, "rate", &dest);
+  }
+}
+
+static GstCaps *gst_audioscale_getcaps (GstPad * pad)
 {
   Audioscale *audioscale;
-  GstPad *otherpad;
-  GstCaps *othercaps;
   GstCaps *caps;
+  GstPad *otherpad;
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
 
   otherpad = (pad == audioscale->srcpad) ? audioscale->sinkpad :
       audioscale->srcpad;
-  othercaps = gst_pad_get_allowed_caps (otherpad);
-  caps = gst_audioscale_expand_caps (othercaps);
-  gst_caps_free (othercaps);
+  caps = gst_pad_get_allowed_caps (otherpad);
+
+  gst_audioscale_expand_caps (caps);
 
   return caps;
 }
 
-static GstCaps *
-gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
+static GstCaps *gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
 {
   Audioscale *audioscale;
   gst_resample_t *r;
@@ -245,12 +310,14 @@ gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
   GstCaps *copy;
   GstStructure *structure;
 
-  audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
-  r = &(audioscale->gst_resample_template);
-  if (pad == audioscale->srcpad) {
+    audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
+    r = &(audioscale->gst_resample_template);
+  if (pad == audioscale->srcpad)
+  {
     otherpad = audioscale->sinkpad;
     rate = r->i_rate;
-  } else {
+  } else
+  {
     otherpad = audioscale->srcpad;
     rate = r->o_rate;
   }
@@ -267,8 +334,7 @@ gst_audioscale_fixate (GstPad * pad, const GstCaps * caps)
   return NULL;
 }
 
-static GstPadLinkReturn
-gst_audioscale_link (GstPad * pad, const GstCaps * caps)
+static GstPadLinkReturn gst_audioscale_link (GstPad * pad, const GstCaps * caps)
 {
   Audioscale *audioscale;
   gst_resample_t *r;
@@ -282,14 +348,16 @@ gst_audioscale_link (GstPad * pad, const GstCaps * caps)
   GstPad *otherpad;
   GstCaps *copy;
 
-  audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
-  r = &(audioscale->gst_resample_template);
+    audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
+    r = &(audioscale->gst_resample_template);
 
-  if (pad == audioscale->srcpad) {
+  if (pad == audioscale->srcpad)
+  {
     otherpad = audioscale->sinkpad;
     rate = &r->o_rate;
     otherrate = &r->i_rate;
-  } else {
+  } else
+  {
     otherpad = audioscale->srcpad;
     rate = &r->i_rate;
     otherrate = &r->o_rate;
@@ -301,9 +369,9 @@ gst_audioscale_link (GstPad * pad, const GstCaps * caps)
   g_return_val_if_fail (ret, GST_PAD_LINK_REFUSED);
   *rate = (double) temp;
 
-  copy = gst_audioscale_expand_caps (caps);
+  copy = gst_caps_copy (caps);
+  gst_audioscale_expand_caps (copy);
   link_ret = gst_pad_try_set_caps_nonfixed (otherpad, copy);
-  gst_caps_free (copy);
   if (GST_PAD_LINK_FAILED (link_ret))
     return link_ret;
 
@@ -380,48 +448,31 @@ gst_audioscale_link (GstPad * pad, const GstCaps * caps)
     GST_DEBUG ("Number of iterations: %d", audioscale->num_iterations);
 
     gst_resample_init (audioscale->gst_resample);
-    audioscale->sample_duration =
-        (GstClockTime) (GST_SECOND / audioscale->gst_resample->o_rate);
-  } else {
-    audioscale->sample_duration = (GstClockTime) (GST_SECOND / r->o_rate);
   }
 
   return link_ret;
 }
 
-static void *
-gst_audioscale_get_buffer (void *priv, unsigned int size)
+static void *gst_audioscale_get_buffer (void *priv, unsigned int size)
 {
   Audioscale *audioscale = priv;
-  gint64 samples;
 
-  GST_DEBUG ("size requested: %u irate: %f orate: %f offset %lld", size,
-      audioscale->gst_resample->i_rate, audioscale->gst_resample->o_rate,
-      audioscale->gst_resample_offset);
+    GST_DEBUG ("size requested: %u irate: %f orate: %f", size,
+      audioscale->gst_resample->i_rate, audioscale->gst_resample->o_rate);
+    audioscale->outbuf = gst_buffer_new ();
+    GST_BUFFER_SIZE (audioscale->outbuf) = size;
+    GST_BUFFER_DATA (audioscale->outbuf) = g_malloc (size);
+    GST_BUFFER_TIMESTAMP (audioscale->outbuf) =
+      audioscale->gst_resample_offset * GST_SECOND /
+      audioscale->gst_resample->o_rate;
+    audioscale->gst_resample_offset +=
+      size / sizeof (gint16) / audioscale->gst_resample->channels;
 
-  audioscale->outbuf = gst_buffer_new ();
-  GST_BUFFER_SIZE (audioscale->outbuf) = size;
-  GST_BUFFER_DATA (audioscale->outbuf) = g_malloc (size);
-  GST_BUFFER_OFFSET (audioscale->outbuf) = audioscale->gst_resample_offset;
-
-  GST_BUFFER_TIMESTAMP (audioscale->outbuf) =
-      audioscale->gst_resample_offset * audioscale->sample_duration;
-
-  samples =
-      GST_BUFFER_SIZE (audioscale->outbuf) / (sizeof (gint16) *
-      audioscale->gst_resample->channels);
-
-  GST_BUFFER_DURATION (audioscale->outbuf) =
-      samples * audioscale->sample_duration;
-
-  audioscale->gst_resample_offset += samples;
-
-  return GST_BUFFER_DATA (audioscale->outbuf);
+    return GST_BUFFER_DATA (audioscale->outbuf);
 }
 
 /* reduces rate by factor of 2 */
-GstBuffer *
-gst_audioscale_decrease_rate (Audioscale * audioscale,
+GstBuffer *gst_audioscale_decrease_rate (Audioscale * audioscale,
     GstBuffer * buf, double outrate, int cur_iteration)
 {
   gint i, j, curoffset;
@@ -449,9 +500,8 @@ gst_audioscale_decrease_rate (Audioscale * audioscale,
   }
 
   GST_BUFFER_DATA (outbuf) = (gpointer) outdata;
-  GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
-  GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buf);
-
+  GST_BUFFER_TIMESTAMP (outbuf) =
+      audioscale->offsets[cur_iteration] * GST_SECOND / outrate;
   audioscale->offsets[cur_iteration] +=
       GST_BUFFER_SIZE (outbuf) / sizeof (gint16) /
       audioscale->gst_resample->channels;
@@ -459,8 +509,7 @@ gst_audioscale_decrease_rate (Audioscale * audioscale,
 }
 
 /* increases rate by factor of 2 */
-GstBuffer *
-gst_audioscale_increase_rate (Audioscale * audioscale,
+GstBuffer *gst_audioscale_increase_rate (Audioscale * audioscale,
     GstBuffer * buf, double outrate, int cur_iteration)
 {
   gint i, j, curoffset;
@@ -489,17 +538,15 @@ gst_audioscale_increase_rate (Audioscale * audioscale,
   }
 
   GST_BUFFER_DATA (outbuf) = (gpointer) outdata;
-  GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
-  GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buf);
-
+  GST_BUFFER_TIMESTAMP (outbuf) =
+      audioscale->offsets[cur_iteration] * GST_SECOND / outrate;
   audioscale->offsets[cur_iteration] +=
       GST_BUFFER_SIZE (outbuf) / sizeof (gint16) /
       audioscale->gst_resample->channels;
   return outbuf;
 }
 
-static void
-gst_audioscale_init (Audioscale * audioscale)
+static void gst_audioscale_init (Audioscale * audioscale)
 {
   gst_resample_t *r;
 
@@ -540,29 +587,21 @@ gst_audioscale_init (Audioscale * audioscale)
   audioscale->offsets = NULL;
   audioscale->gst_resample_offset = 0;
   audioscale->increase = FALSE;
-
-  GST_FLAG_SET (audioscale, GST_ELEMENT_EVENT_AWARE);
 }
 
-static void
-gst_audioscale_dispose (GObject * object)
+static void gst_audioscale_dispose (GObject * object)
 {
   Audioscale *audioscale = GST_AUDIOSCALE (object);
 
   if (audioscale->gst_resample) {
-    gst_resample_close (audioscale->gst_resample);
     g_free (audioscale->gst_resample);
-    audioscale->gst_resample = NULL;
   }
-  if (audioscale->offsets) {
+  if (audioscale->offsets)
     g_free (audioscale->offsets);
-    audioscale->offsets = NULL;
-  }
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static void
-gst_audioscale_chain (GstPad * pad, GstData * _data)
+static void gst_audioscale_chain (GstPad * pad, GstData * _data)
 {
   GstBuffer *buf = GST_BUFFER (_data);
   GstBuffer *tempbuf, *tempbuf2;
@@ -578,85 +617,6 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
   g_return_if_fail (buf != NULL);
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
-
-  if (audioscale->gst_resample && audioscale->saved_discont) {
-    GstData *saved_event = GST_DATA (audioscale->saved_discont);
-
-    audioscale->saved_discont = NULL;
-    GST_DEBUG ("processing saved discont event");
-    gst_audioscale_chain (pad, saved_event);
-  }
-
-  if (GST_IS_EVENT (_data)) {
-    GstEvent *e = GST_EVENT (_data);
-
-    switch (GST_EVENT_TYPE (e)) {
-      case GST_EVENT_DISCONTINUOUS:{
-        gint64 ts = (gint64) GST_CLOCK_TIME_NONE;
-        gint64 off = (gint64) GST_BUFFER_OFFSET_NONE;
-
-        if (audioscale->passthru) {
-          gst_pad_event_default (pad, e);
-          return;
-        } else if (!audioscale->gst_resample) {
-          if (audioscale->saved_discont)
-            gst_data_unref (GST_DATA (audioscale->saved_discont));
-          audioscale->saved_discont = e;
-          GST_DEBUG ("Discont before negotiation happened, will process later");
-          return;               /* process it later when we're set up */
-        } else if (gst_event_discont_get_value (e, GST_FORMAT_TIME, &ts)) {
-          GST_DEBUG ("time discont [%" GST_TIME_FORMAT "]", GST_TIME_ARGS (ts));
-          /* time -> out-sample */
-          off = ts / audioscale->sample_duration;
-          ts = off * audioscale->sample_duration;
-        } else if (gst_event_discont_get_value (e, GST_FORMAT_DEFAULT, &off)) {
-          GST_DEBUG ("default discont");
-          /* in-sample -> out-sample */
-          off *= audioscale->gst_resample->o_rate;
-          off /= audioscale->gst_resample->i_rate;
-          ts = off * audioscale->sample_duration;
-        } else if (gst_event_discont_get_value (e, GST_FORMAT_BYTES, &off)) {
-          GST_DEBUG ("Discont event, bytes = %" G_GUINT64_FORMAT, off);
-          off /= audioscale->gst_resample->channels;
-          off /= (audioscale->gst_resample->format == GST_RESAMPLE_S16) ? 2 : 4;
-          off *= audioscale->gst_resample->o_rate;
-          off /= audioscale->gst_resample->i_rate;
-          ts = off * audioscale->sample_duration;
-        } else {
-          /* *sigh* */
-          GST_DEBUG ("Discont without value - ignoring");
-          gst_data_unref (_data);
-          return;
-        }
-        audioscale->gst_resample_offset = off;
-
-        GST_DEBUG ("New resample offset after discont event: %"
-            G_GUINT64_FORMAT ", %" GST_TIME_FORMAT, off, GST_TIME_ARGS (ts));
-
-        e = gst_event_new_discontinuous (GST_EVENT_DISCONT_NEW_MEDIA (e),
-            GST_FORMAT_DEFAULT, off, GST_FORMAT_TIME, ts, GST_FORMAT_UNDEFINED);
-        gst_pad_event_default (pad, e);
-        gst_data_unref (_data);
-        return;
-      }
-        break;
-      default:
-        gst_pad_event_default (pad, e);
-        return;
-        break;
-    }
-  }
-
-  if (GST_BUFFER_TIMESTAMP_IS_VALID (buf) && audioscale->gst_resample) {
-    GST_DEBUG ("input buffer timestamp [%" GST_TIME_FORMAT "] -> [%"
-        GST_TIME_FORMAT "] %d samples",
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf)),
-        GST_BUFFER_SIZE (buf) / (sizeof (gint16) *
-            audioscale->gst_resample->channels)
-        );
-  }
-
   if (audioscale->passthru && audioscale->num_iterations == 0) {
     gst_pad_push (audioscale->srcpad, GST_DATA (buf));
     return;
@@ -676,7 +636,6 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
     tempbuf = audioscale->outbuf;
     gst_buffer_unref (buf);
     outrate = audioscale->gst_resample->o_rate;
-
   }
   for (i = 0; i < audioscale->num_iterations; i++) {
     tempbuf2 = tempbuf;
@@ -702,41 +661,21 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
     gst_buffer_unref (tempbuf);
     tempbuf = audioscale->outbuf;
   }
+  gst_pad_push (audioscale->srcpad, GST_DATA (tempbuf));
 
-  if (GST_BUFFER_SIZE (tempbuf) > 0)
-    gst_pad_push (audioscale->srcpad, GST_DATA (tempbuf));
-  else
-    gst_buffer_unref (tempbuf);
-
-}
-
-static GstElementStateReturn
-gst_audioscale_change_state (GstElement * element)
-{
-  Audioscale *audioscale = GST_AUDIOSCALE (element);
-
-  switch (GST_STATE_TRANSITION (element)) {
-    case GST_STATE_PAUSED_TO_READY:
-      audioscale->gst_resample_offset = 0;
-      break;
-    default:
-      break;
-  }
-
-  return parent_class->change_state (element);
 }
 
 static void
-gst_audioscale_set_property (GObject * object, guint prop_id,
+    gst_audioscale_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   Audioscale *src;
   gst_resample_t *r;
 
   /* it's not null if we got it, but it might not be ours */
-  g_return_if_fail (GST_IS_AUDIOSCALE (object));
-  src = GST_AUDIOSCALE (object);
-  r = &(src->gst_resample_template);
+    g_return_if_fail (GST_IS_AUDIOSCALE (object));
+    src = GST_AUDIOSCALE (object);
+    r = &(src->gst_resample_template);
 
   switch (prop_id) {
     case ARG_FILTERLEN:
@@ -744,11 +683,9 @@ gst_audioscale_set_property (GObject * object, guint prop_id,
       GST_DEBUG_OBJECT (GST_ELEMENT (src), "new filter length %d\n",
           r->filter_length);
       break;
-    case ARG_METHOD:
-      r->method = g_value_get_enum (value);
+      case ARG_METHOD:r->method = g_value_get_enum (value);
       break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      default:G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 
@@ -756,7 +693,7 @@ gst_audioscale_set_property (GObject * object, guint prop_id,
 }
 
 static void
-gst_audioscale_get_property (GObject * object, guint prop_id,
+    gst_audioscale_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
   Audioscale *src;
@@ -779,8 +716,7 @@ gst_audioscale_get_property (GObject * object, guint prop_id,
 }
 
 
-static gboolean
-plugin_init (GstPlugin * plugin)
+static gboolean plugin_init (GstPlugin * plugin)
 {
   /* load support library */
   if (!gst_library_load ("gstresample"))
