@@ -21,7 +21,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <gstrtjpegenc.h>
+#include "gstrtjpegenc.h"
 
 /* elementfactory information */
 GstElementDetails gst_rtjpegenc_details = {
@@ -41,7 +41,7 @@ enum
 enum
 {
   ARG_0,
-  ARG_QUALITY,
+  ARG_QUALITY
 };
 
 
@@ -72,6 +72,7 @@ gst_rtjpegenc_get_type (void)
       0,
       (GInstanceInitFunc) gst_rtjpegenc_init,
     };
+
     rtjpegenc_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstRTJpegEnc",
         &rtjpegenc_info, 0);
@@ -123,7 +124,7 @@ gst_rtjpegenc_chain (GstPad * pad, GstData * _data)
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
 
-  gst_info ("would be encoding frame here\n");
+  g_warning ("would be encoding frame here\n");
 
   gst_pad_push (rtjpegenc->srcpad, GST_DATA (buf));
 }
