@@ -1,4 +1,7 @@
-/* G-Streamer BT8x8/V4L frame grabber plugin
+/* GStreamer
+ *
+ * gstv4lsrc.h: BT8x8/V4L video source element
+ *
  * Copyright (C) 2001-2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -23,7 +26,6 @@
 #include <gstv4lelement.h>
 
 G_BEGIN_DECLS
-
 #define GST_TYPE_V4LSRC \
   (gst_v4lsrc_get_type())
 #define GST_V4LSRC(obj) \
@@ -34,19 +36,32 @@ G_BEGIN_DECLS
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_V4LSRC))
 #define GST_IS_V4LSRC_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_V4LSRC))
-
 typedef struct _GstV4lSrc GstV4lSrc;
 typedef struct _GstV4lSrcClass GstV4lSrcClass;
 
-struct _GstV4lSrc {
+enum
+{
+  QUEUE_STATE_ERROR = -1,
+  QUEUE_STATE_READY_FOR_QUEUE, /* the frame is ready to be queued for capture */
+  QUEUE_STATE_QUEUED,          /* the frame is queued for capture */
+  QUEUE_STATE_SYNCED           /* the frame is captured */
+};
+
+typedef enum
+{
+  GST_V4LSRC_SYNC_MODE_CLOCK,
+  GST_V4LSRC_SYNC_MODE_PRIVATE_CLOCK,
+  GST_V4LSRC_SYNC_MODE_FIXED_FPS,
+  GST_V4LSRC_SYNC_MODE_NONE,
+} GstV4lSrcSyncMode;
+
+struct _GstV4lSrc
+{
   GstV4lElement v4lelement;
 
   /* pads */
   GstPad *srcpad;
 
-  /* bufferpool for the buffers we're gonna use */
-  GstBufferPool *bufferpool;
-
   /* capture/buffer info */
   struct video_mmap mmap;
   struct video_mbuf mbuf;
@@ -60,6 +75,8 @@ struct _GstV4lSrc {
   GCond *cond_queue_state;
   gint num_queued;
   gint sync_frame, queue_frame;
+  gboolean is_capturing;
+  GstClockTimeDiff latency_offset;
 
   /* True if we want to stop */
   gboolean quit;
@@ -68,6 +85,8 @@ struct _GstV4lSrc {
   gulong handled;
   gint last_frame;
   gint need_writes;
+  gboolean need_discont;
+  GstClockTime last_discont;    /* gst_element_get_time () of last discont send */
 
   /* clock */
   GstClock *clock;
@@ -78,21 +97,27 @@ struct _GstV4lSrc {
   /* how often are we going to use each frame? */
   gint *use_num_times;
 
-  /* how are we going to push buffers? */
-  gboolean use_fixed_fps;
+  /* list of supported colourspaces (as integers) */
+  GList *colourspaces;
+
+  /* how are we going to timestamp buffers? */
+   GstV4lSrcSyncMode syncmode;
+
+   gboolean copy_mode;
+   gboolean autoprobe; /* probe on startup ? */
 };
 
-struct _GstV4lSrcClass {
+struct _GstV4lSrcClass
+{
   GstV4lElementClass parent_class;
 
-  void (*frame_capture) (GObject *object);
-  void (*frame_drop)    (GObject *object);
-  void (*frame_insert)  (GObject *object);
+  void (*frame_capture) (GObject * object);
+  void (*frame_drop) (GObject * object);
+  void (*frame_insert) (GObject * object);
 };
 
-GType gst_v4lsrc_get_type(void);
+GType gst_v4lsrc_get_type (void);
 
 
 G_END_DECLS
-
 #endif /* __GST_V4LSRC_H__ */
