@@ -33,7 +33,21 @@
 GST_DEBUG_CATEGORY_STATIC (gst_debug_ximagesink);
 #define GST_CAT_DEFAULT gst_debug_ximagesink
 
+typedef struct
+{
+  unsigned long flags;
+  unsigned long functions;
+  unsigned long decorations;
+  long input_mode;
+  unsigned long status;
+}
+MotifWmHints, MwmHints;
+
+#define MWM_HINTS_DECORATIONS   (1L << 1)
+
 static void gst_ximagesink_buffer_free (GstBuffer * buffer);
+static void gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink,
+    GstXImage * ximage);
 
 /* ElementFactory information */
 static GstElementDetails gst_ximagesink_details =
@@ -57,12 +71,13 @@ enum
 {
   ARG_0,
   ARG_DISPLAY,
-  ARG_SYNCHRONOUS
+  ARG_SYNCHRONOUS,
+  ARG_PIXEL_ASPECT_RATIO
       /* FILL ME */
 };
 
 static GstVideoSinkClass *parent_class = NULL;
-static gboolean error_catched = FALSE;
+static gboolean error_caught = FALSE;
 
 /* ============================================================= */
 /*                                                               */
@@ -79,7 +94,7 @@ gst_ximagesink_handle_xerror (Display * display, XErrorEvent * xevent)
 
   XGetErrorText (display, xevent->error_code, error_msg, 1024);
   GST_DEBUG ("ximagesink failed to use XShm calls. error: %s", error_msg);
-  error_catched = TRUE;
+  error_caught = TRUE;
   return 0;
 }
 
@@ -88,56 +103,76 @@ gst_ximagesink_handle_xerror (Display * display, XErrorEvent * xevent)
 static gboolean
 gst_ximagesink_check_xshm_calls (GstXContext * xcontext)
 {
+#ifndef HAVE_XSHM
+  return FALSE;
+#else
   GstXImage *ximage = NULL;
   int (*handler) (Display *, XErrorEvent *);
+  gboolean result = FALSE;
 
   g_return_val_if_fail (xcontext != NULL, FALSE);
 
-#ifdef HAVE_XSHM
   ximage = g_new0 (GstXImage, 1);
+  g_return_val_if_fail (ximage != NULL, FALSE);
 
   /* Setting an error handler to catch failure */
+  error_caught = FALSE;
   handler = XSetErrorHandler (gst_ximagesink_handle_xerror);
 
-  ximage->size = (xcontext->bpp / 8);
-
   /* Trying to create a 1x1 picture */
+  GST_DEBUG ("XShmCreateImage of 1x1");
+
   ximage->ximage = XShmCreateImage (xcontext->disp, xcontext->visual,
       xcontext->depth, ZPixmap, NULL, &ximage->SHMInfo, 1, 1);
+  if (!ximage->ximage) {
+    GST_WARNING ("could not XShmCreateImage a 1x1 image");
+    goto beach;
+  }
+  ximage->size = ximage->ximage->height * ximage->ximage->bytes_per_line;
 
   ximage->SHMInfo.shmid = shmget (IPC_PRIVATE, ximage->size, IPC_CREAT | 0777);
+  if (ximage->SHMInfo.shmid == -1) {
+    GST_WARNING ("could not get shared memory of %d bytes", ximage->size);
+    goto beach;
+  }
+
   ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, 0, 0);
+  if (ximage->SHMInfo.shmaddr == ((void *) -1)) {
+    GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
+    goto beach;
+  }
+
   ximage->ximage->data = ximage->SHMInfo.shmaddr;
   ximage->SHMInfo.readOnly = FALSE;
 
-  XShmAttach (xcontext->disp, &ximage->SHMInfo);
-
-  error_catched = FALSE;
+  if (XShmAttach (xcontext->disp, &ximage->SHMInfo) == 0) {
+    GST_WARNING ("Failed to XShmAttach");
+    goto beach;
+  }
 
   XSync (xcontext->disp, 0);
 
+  XShmDetach (xcontext->disp, &ximage->SHMInfo);
+  XSync (xcontext->disp, FALSE);
+
+  shmdt (ximage->SHMInfo.shmaddr);
+  shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
+
+  /* To be sure, reset the SHMInfo entry */
+  ximage->SHMInfo.shmaddr = ((void *) -1);
+
+  /* store whether we succeeded in result and reset error_caught */
+  result = !error_caught;
+  error_caught = FALSE;
+
+beach:
   XSetErrorHandler (handler);
-
-  if (error_catched) {          /* Failed, detaching shared memory, destroying image and telling we can't
-                                   use XShm */
-    error_catched = FALSE;
-    XDestroyImage (ximage->ximage);
-    shmdt (ximage->SHMInfo.shmaddr);
-    shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
-    g_free (ximage);
-    XSync (xcontext->disp, FALSE);
-    return FALSE;
-  } else {
-    XShmDetach (xcontext->disp, &ximage->SHMInfo);
-    XDestroyImage (ximage->ximage);
-    shmdt (ximage->SHMInfo.shmaddr);
-    shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
-    g_free (ximage);
-    XSync (xcontext->disp, FALSE);
-  }
+  if (ximage->ximage)
+    XFree (ximage->ximage);
+  g_free (ximage);
+  XSync (xcontext->disp, FALSE);
+  return result;
 #endif /* HAVE_XSHM */
-
-  return TRUE;
 }
 
 /* This function handles GstXImage creation depending on XShm availability */
@@ -145,69 +180,90 @@ static GstXImage *
 gst_ximagesink_ximage_new (GstXImageSink * ximagesink, gint width, gint height)
 {
   GstXImage *ximage = NULL;
+  gboolean succeeded = FALSE;
 
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
+  GST_DEBUG_OBJECT (ximagesink, "creating %dx%d", width, height);
 
   ximage = g_new0 (GstXImage, 1);
 
   ximage->width = width;
   ximage->height = height;
-  ximage->data = NULL;
   ximage->ximagesink = ximagesink;
 
   g_mutex_lock (ximagesink->x_lock);
 
-  ximage->size =
-      (ximagesink->xcontext->bpp / 8) * ximage->width * ximage->height;
-
 #ifdef HAVE_XSHM
   if (ximagesink->xcontext->use_xshm) {
     ximage->ximage = XShmCreateImage (ximagesink->xcontext->disp,
         ximagesink->xcontext->visual,
         ximagesink->xcontext->depth,
         ZPixmap, NULL, &ximage->SHMInfo, ximage->width, ximage->height);
+    if (!ximage->ximage) {
+      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
+          ("could not XShmCreateImage a %dx%d image"));
+      goto beach;
+    }
+
+    /* we have to use the returned bytes_per_line for our shm size */
+    ximage->size = ximage->ximage->bytes_per_line * ximage->ximage->height;
+    GST_DEBUG_OBJECT (ximagesink, "XShm image size is %d, width %d, stride %d",
+        ximage->size, ximage->width, ximage->ximage->bytes_per_line);
 
     ximage->SHMInfo.shmid = shmget (IPC_PRIVATE, ximage->size,
         IPC_CREAT | 0777);
+    if (ximage->SHMInfo.shmid == -1) {
+      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
+          ("could not get shared memory of %d bytes", ximage->size));
+      goto beach;
+    }
 
     ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, 0, 0);
-    ximage->ximage->data = ximage->SHMInfo.shmaddr;
+    if (ximage->SHMInfo.shmaddr == ((void *) -1)) {
+      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
+          ("Failed to shmat: %s", g_strerror (errno)));
+      goto beach;
+    }
 
+    ximage->ximage->data = ximage->SHMInfo.shmaddr;
     ximage->SHMInfo.readOnly = FALSE;
 
-    XShmAttach (ximagesink->xcontext->disp, &ximage->SHMInfo);
+    if (XShmAttach (ximagesink->xcontext->disp, &ximage->SHMInfo) == 0) {
+      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
+          ("Failed to XShmAttach"));
+      goto beach;
+    }
 
     XSync (ximagesink->xcontext->disp, FALSE);
-
-    shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
-    ximage->SHMInfo.shmid = -1;
   } else
 #endif /* HAVE_XSHM */
   {
-    ximage->data = g_malloc (ximage->size);
-
     ximage->ximage = XCreateImage (ximagesink->xcontext->disp,
         ximagesink->xcontext->visual,
         ximagesink->xcontext->depth,
-        ZPixmap, 0, ximage->data,
-        ximage->width, ximage->height,
-        ximagesink->xcontext->bpp,
-        ximage->width * (ximagesink->xcontext->bpp / 8));
+        ZPixmap, 0, NULL,
+        ximage->width, ximage->height, ximagesink->xcontext->bpp, 0);
+    if (!ximage->ximage) {
+      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
+          ("could not XCreateImage a %dx%d image"));
+      goto beach;
+    }
+
+    /* we have to use the returned bytes_per_line for our image size */
+    ximage->size = ximage->ximage->bytes_per_line * ximage->ximage->height;
+    ximage->ximage->data = g_malloc (ximage->size);
 
     XSync (ximagesink->xcontext->disp, FALSE);
   }
+  succeeded = TRUE;
+  g_mutex_unlock (ximagesink->x_lock);
 
-  if (!ximage->ximage) {
-    if (ximage->data)
-      g_free (ximage->data);
-
-    g_free (ximage);
-
+beach:
+  if (!succeeded) {
+    gst_ximagesink_ximage_destroy (ximagesink, ximage);
     ximage = NULL;
   }
 
-  g_mutex_unlock (ximagesink->x_lock);
-
   return ximage;
 }
 
@@ -226,22 +282,22 @@ gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink, GstXImage * ximage)
 
 #ifdef HAVE_XSHM
   if (ximagesink->xcontext->use_xshm) {
-    if (ximage->SHMInfo.shmaddr)
+    if (ximage->SHMInfo.shmaddr != ((void *) -1)) {
       XShmDetach (ximagesink->xcontext->disp, &ximage->SHMInfo);
-
+      XSync (ximagesink->xcontext->disp, 0);
+      shmdt (ximage->SHMInfo.shmaddr);
+    }
+    if (ximage->SHMInfo.shmid > 0)
+      shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
     if (ximage->ximage)
       XDestroyImage (ximage->ximage);
 
-    if (ximage->SHMInfo.shmaddr)
-      shmdt (ximage->SHMInfo.shmaddr);
-
-    if (ximage->SHMInfo.shmid > 0)
-      shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
   } else
 #endif /* HAVE_XSHM */
   {
-    if (ximage->ximage)
+    if (ximage->ximage) {
       XDestroyImage (ximage->ximage);
+    }
   }
 
   XSync (ximagesink->xcontext->disp, FALSE);
@@ -256,6 +312,7 @@ static void
 gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImage * ximage)
 {
   gint x, y;
+  gint w, h;
 
   g_return_if_fail (ximage != NULL);
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
@@ -264,22 +321,26 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImage * ximage)
   if (ximagesink->cur_image != ximage)
     ximagesink->cur_image = ximage;
 
-  /* We center the image in the window */
+  /* We center the image in the window; so calculate top left corner location */
   x = MAX (0, (ximagesink->xwindow->width - ximage->width) / 2);
   y = MAX (0, (ximagesink->xwindow->height - ximage->height) / 2);
 
+  w = ximage->width;
+  h = ximage->height;
+
   g_mutex_lock (ximagesink->x_lock);
 #ifdef HAVE_XSHM
   if (ximagesink->xcontext->use_xshm) {
+    GST_LOG_OBJECT (ximagesink,
+        "XShmPutImage, src: %d, %d - dest: %d, %d, dim: %dx%d",
+        0, 0, x, y, w, h);
     XShmPutImage (ximagesink->xcontext->disp, ximagesink->xwindow->win,
-        ximagesink->xwindow->gc, ximage->ximage,
-        0, 0, x, y, ximage->width, ximage->height, FALSE);
+        ximagesink->xwindow->gc, ximage->ximage, 0, 0, x, y, w, h, FALSE);
   } else
 #endif /* HAVE_XSHM */
   {
     XPutImage (ximagesink->xcontext->disp, ximagesink->xwindow->win,
-        ximagesink->xwindow->gc, ximage->ximage,
-        0, 0, x, y, ximage->width, ximage->height);
+        ximagesink->xwindow->gc, ximage->ximage, 0, 0, x, y, w, h);
   }
 
   XSync (ximagesink->xcontext->disp, FALSE);
@@ -287,6 +348,42 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImage * ximage)
   g_mutex_unlock (ximagesink->x_lock);
 }
 
+static gboolean
+gst_ximagesink_xwindow_decorate (GstXImageSink * ximagesink,
+    GstXWindow * window)
+{
+  Atom hints_atom = None;
+  MotifWmHints *hints;
+
+  g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), FALSE);
+  g_return_val_if_fail (window != NULL, FALSE);
+
+  g_mutex_lock (ximagesink->x_lock);
+
+  hints_atom = XInternAtom (ximagesink->xcontext->disp, "_MOTIF_WM_HINTS", 1);
+  if (hints_atom == None) {
+    g_mutex_unlock (ximagesink->x_lock);
+    return FALSE;
+  }
+
+  hints = g_malloc0 (sizeof (MotifWmHints));
+
+  hints->flags |= MWM_HINTS_DECORATIONS;
+  hints->decorations = 1 << 0;
+
+  XChangeProperty (ximagesink->xcontext->disp, window->win,
+      hints_atom, hints_atom, 32, PropModeReplace,
+      (guchar *) hints, sizeof (MotifWmHints) / sizeof (long));
+
+  XSync (ximagesink->xcontext->disp, FALSE);
+
+  g_mutex_unlock (ximagesink->x_lock);
+
+  g_free (hints);
+
+  return TRUE;
+}
+
 /* This function handles a GstXWindow creation */
 static GstXWindow *
 gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
@@ -319,6 +416,8 @@ gst_ximagesink_xwindow_new (GstXImageSink * ximagesink, gint width, gint height)
 
   g_mutex_unlock (ximagesink->x_lock);
 
+  gst_ximagesink_xwindow_decorate (ximagesink, xwindow);
+
   gst_x_overlay_got_xwindow_id (GST_X_OVERLAY (ximagesink), xwindow->win);
 
   return xwindow;
@@ -435,18 +534,29 @@ gst_ximagesink_renegotiate_size (GstXImageSink * ximagesink)
   if (GST_VIDEOSINK_WIDTH (ximagesink) != ximagesink->xwindow->width ||
       GST_VIDEOSINK_HEIGHT (ximagesink) != ximagesink->xwindow->height) {
     GstPadLinkReturn r;
+    GstCaps *caps;
 
-    r = gst_pad_try_set_caps (GST_VIDEOSINK_PAD (ximagesink),
-        gst_caps_new_simple ("video/x-raw-rgb",
-            "bpp", G_TYPE_INT, ximagesink->xcontext->bpp,
-            "depth", G_TYPE_INT, ximagesink->xcontext->depth,
-            "endianness", G_TYPE_INT, ximagesink->xcontext->endianness,
-            "red_mask", G_TYPE_INT, ximagesink->xcontext->visual->red_mask,
-            "green_mask", G_TYPE_INT, ximagesink->xcontext->visual->green_mask,
-            "blue_mask", G_TYPE_INT, ximagesink->xcontext->visual->blue_mask,
-            "width", G_TYPE_INT, ximagesink->xwindow->width,
-            "height", G_TYPE_INT, ximagesink->xwindow->height,
-            "framerate", G_TYPE_DOUBLE, ximagesink->framerate, NULL));
+    caps = gst_caps_new_simple ("video/x-raw-rgb",
+        "bpp", G_TYPE_INT, ximagesink->xcontext->bpp,
+        "depth", G_TYPE_INT, ximagesink->xcontext->depth,
+        "endianness", G_TYPE_INT, ximagesink->xcontext->endianness,
+        "red_mask", G_TYPE_INT, ximagesink->xcontext->visual->red_mask,
+        "green_mask", G_TYPE_INT, ximagesink->xcontext->visual->green_mask,
+        "blue_mask", G_TYPE_INT, ximagesink->xcontext->visual->blue_mask,
+        "width", G_TYPE_INT, ximagesink->xwindow->width,
+        "height", G_TYPE_INT, ximagesink->xwindow->height,
+        "framerate", G_TYPE_DOUBLE, ximagesink->framerate, NULL);
+
+    if (ximagesink->par) {
+      int nom, den;
+
+      nom = gst_value_get_fraction_numerator (ximagesink->par);
+      den = gst_value_get_fraction_denominator (ximagesink->par);
+      gst_caps_set_simple (caps, "pixel-aspect-ratio", GST_TYPE_FRACTION,
+          nom, den, NULL);
+    }
+
+    r = gst_pad_try_set_caps (GST_VIDEOSINK_PAD (ximagesink), caps);
 
     if ((r == GST_PAD_LINK_OK) || (r == GST_PAD_LINK_DONE)) {
       /* Renegotiation succeeded, we update our size and image */
@@ -458,11 +568,9 @@ gst_ximagesink_renegotiate_size (GstXImageSink * ximagesink)
               (GST_VIDEOSINK_HEIGHT (ximagesink) !=
                   ximagesink->ximage->height))) {
         /* We renew our ximage only if size changed */
+        GST_DEBUG_OBJECT (ximagesink, "destroying and recreating our ximage");
         gst_ximagesink_ximage_destroy (ximagesink, ximagesink->ximage);
-
-        ximagesink->ximage = gst_ximagesink_ximage_new (ximagesink,
-            GST_VIDEOSINK_WIDTH (ximagesink),
-            GST_VIDEOSINK_HEIGHT (ximagesink));
+        ximagesink->ximage = NULL;
       }
     } else {
       ximagesink->sw_scaling_failed = TRUE;
@@ -559,15 +667,69 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink, GstPad * pad)
       default:
         GST_DEBUG ("ximagesink unhandled X event (%d)", e.type);
     }
-
     g_mutex_lock (ximagesink->x_lock);
   }
   g_mutex_unlock (ximagesink->x_lock);
 }
 
-/* This function get the X Display and global infos about it. Everything is
+/* This function calculates the pixel aspect ratio based on the properties
+ * in the xcontext structure and stores it there. */
+static void
+gst_ximagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
+{
+  gint par[][2] = {
+    {1, 1},                     /* regular screen */
+    {16, 15},                   /* PAL TV */
+    {11, 10},                   /* 525 line Rec.601 video */
+    {54, 59}                    /* 625 line Rec.601 video */
+  };
+  gint i;
+  gint index;
+  gdouble ratio;
+  gdouble delta;
+
+#define DELTA(idx) (ABS (ratio - ((gdouble) par[idx][0] / par[idx][1])))
+
+  /* first calculate the "real" ratio based on the X values;
+   * which is the "physical" w/h divided by the w/h in pixels of the display */
+  ratio = (gdouble) (xcontext->widthmm * xcontext->height)
+      / (xcontext->heightmm * xcontext->width);
+
+  /* DirectFB's X in 720x576 reports the physical dimensions wrong, so
+   * override here */
+  if (xcontext->width == 720 && xcontext->height == 576) {
+    ratio = 4.0 * 576 / (3.0 * 720);
+  }
+  GST_DEBUG ("calculated pixel aspect ratio: %f", ratio);
+
+  /* now find the one from par[][2] with the lowest delta to the real one */
+  delta = DELTA (0);
+  index = 0;
+
+  for (i = 1; i < sizeof (par) / (sizeof (gint) * 2); ++i) {
+    gdouble this_delta = DELTA (i);
+
+    if (this_delta < delta) {
+      index = i;
+      delta = this_delta;
+    }
+  }
+
+  GST_DEBUG ("Decided on index %d (%d/%d)", index,
+      par[index][0], par[index][1]);
+
+  g_free (xcontext->par);
+  xcontext->par = g_new0 (GValue, 1);
+  g_value_init (xcontext->par, GST_TYPE_FRACTION);
+  gst_value_set_fraction (xcontext->par, par[index][0], par[index][1]);
+  GST_DEBUG ("set xcontext PAR to %d/%d",
+      gst_value_get_fraction_numerator (xcontext->par),
+      gst_value_get_fraction_denominator (xcontext->par));
+}
+
+/* This function gets the X Display and global info about it. Everything is
    stored in our object and will be cleaned when the object is disposed. Note
-   here that caps for supported format are generated without any window or 
+   here that caps for supported format are generated without any window or
    image creation */
 static GstXContext *
 gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
@@ -587,7 +749,7 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
   if (!xcontext->disp) {
     g_mutex_unlock (ximagesink->x_lock);
     g_free (xcontext);
-    GST_ELEMENT_ERROR (ximagesink, RESOURCE, TOO_LAZY, (NULL),
+    GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
         ("Could not open display"));
     return NULL;
   }
@@ -600,6 +762,16 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
   xcontext->black = XBlackPixel (xcontext->disp, xcontext->screen_num);
   xcontext->depth = DefaultDepthOfScreen (xcontext->screen);
 
+  xcontext->width = DisplayWidth (xcontext->disp, xcontext->screen_num);
+  xcontext->height = DisplayHeight (xcontext->disp, xcontext->screen_num);
+  xcontext->widthmm = DisplayWidthMM (xcontext->disp, xcontext->screen_num);
+  xcontext->heightmm = DisplayHeightMM (xcontext->disp, xcontext->screen_num);
+
+  GST_DEBUG_OBJECT (ximagesink, "X reports %dx%d pixels and %d mm x %d mm",
+      xcontext->width, xcontext->height, xcontext->widthmm, xcontext->heightmm);
+
+  gst_ximagesink_calculate_pixel_aspect_ratio (xcontext);
+
   /* We get supported pixmap formats at supported depth */
   px_formats = XListPixmapFormats (xcontext->disp, &nb_formats);
 
@@ -648,6 +820,12 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
     }
   }
 
+  /* update object's par with calculated one if not set yet */
+  if (!ximagesink->par) {
+    ximagesink->par = g_new0 (GValue, 1);
+    gst_value_init_and_copy (ximagesink->par, xcontext->par);
+    GST_DEBUG_OBJECT (ximagesink, "set calculated PAR on object's PAR");
+  }
   xcontext->caps = gst_caps_new_simple ("video/x-raw-rgb",
       "bpp", G_TYPE_INT, xcontext->bpp,
       "depth", G_TYPE_INT, xcontext->depth,
@@ -658,6 +836,14 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
       "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
       "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
       "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 100.0, NULL);
+  if (ximagesink->par) {
+    int nom, den;
+
+    nom = gst_value_get_fraction_numerator (ximagesink->par);
+    den = gst_value_get_fraction_denominator (ximagesink->par);
+    gst_caps_set_simple (xcontext->caps, "pixel-aspect-ratio",
+        GST_TYPE_FRACTION, nom, den, NULL);
+  }
 
   g_mutex_unlock (ximagesink->x_lock);
 
@@ -672,6 +858,9 @@ gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
 
   gst_caps_free (ximagesink->xcontext->caps);
+  g_free (ximagesink->xcontext->par);
+  g_free (ximagesink->par);
+  ximagesink->par = NULL;
 
   g_mutex_lock (ximagesink->x_lock);
 
@@ -712,6 +901,8 @@ gst_ximagesink_fixate (GstPad * pad, const GstCaps * caps)
   newcaps = gst_caps_copy (caps);
   structure = gst_caps_get_structure (newcaps, 0);
 
+  /* if par is set and either w or h is set, we can set the other */
+
   if (gst_caps_structure_fixate_field_nearest_int (structure, "width", 320)) {
     return newcaps;
   }
@@ -731,15 +922,29 @@ static GstCaps *
 gst_ximagesink_getcaps (GstPad * pad)
 {
   GstXImageSink *ximagesink;
+  GstCaps *caps;
+  int i;
 
   ximagesink = GST_XIMAGESINK (gst_pad_get_parent (pad));
 
   if (ximagesink->xcontext)
     return gst_caps_copy (ximagesink->xcontext->caps);
 
-  return gst_caps_from_string ("video/x-raw-rgb, "
-      "framerate = (double) [ 1, 100 ], "
-      "width = (int) [ 0, MAX ], " "height = (int) [ 0, MAX ]");
+  /* get a template copy and add the pixel aspect ratio */
+  caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+  for (i = 0; i < gst_caps_get_size (caps); ++i) {
+    GstStructure *structure = gst_caps_get_structure (caps, i);
+
+    if (ximagesink->par) {
+      int nom, den;
+
+      nom = gst_value_get_fraction_numerator (ximagesink->par);
+      den = gst_value_get_fraction_denominator (ximagesink->par);
+      gst_structure_set (structure, "pixel-aspect-ratio",
+          GST_TYPE_FRACTION, nom, den, NULL);
+    }
+  }
+  return caps;
 }
 
 static GstPadLinkReturn
@@ -748,6 +953,7 @@ gst_ximagesink_sink_link (GstPad * pad, const GstCaps * caps)
   GstXImageSink *ximagesink;
   gboolean ret;
   GstStructure *structure;
+  const GValue *par;
 
   ximagesink = GST_XIMAGESINK (gst_pad_get_parent (pad));
 
@@ -768,30 +974,37 @@ gst_ximagesink_sink_link (GstPad * pad, const GstCaps * caps)
   if (!ret)
     return GST_PAD_LINK_REFUSED;
 
-  ximagesink->pixel_width = 1;
-  gst_structure_get_int (structure, "pixel_width", &ximagesink->pixel_width);
-
-  ximagesink->pixel_height = 1;
-  gst_structure_get_int (structure, "pixel_height", &ximagesink->pixel_height);
-
-  /* Creating our window and our image */
-  if (!ximagesink->xwindow)
-    ximagesink->xwindow = gst_ximagesink_xwindow_new (ximagesink,
-        GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
-  else {
-    if (ximagesink->xwindow->internal)
-      gst_ximagesink_xwindow_resize (ximagesink, ximagesink->xwindow,
-          GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
+  /* if the caps contain pixel-aspect-ratio, they have to match ours,
+   * otherwise linking should fail */
+  par = gst_structure_get_value (structure, "pixel-aspect-ratio");
+  if (par) {
+    if (gst_value_compare (par, ximagesink->par) != GST_VALUE_EQUAL) {
+      GST_INFO_OBJECT (ximagesink, "pixel aspect ratio does not match");
+      return GST_PAD_LINK_REFUSED;
+    }
   }
 
-  if ((ximagesink->ximage) && ((GST_VIDEOSINK_WIDTH (ximagesink) != ximagesink->ximage->width) || (GST_VIDEOSINK_HEIGHT (ximagesink) != ximagesink->ximage->height))) { /* We renew our ximage only if size changed */
-    gst_ximagesink_ximage_destroy (ximagesink, ximagesink->ximage);
+  /* Creating our window and our image */
+  g_assert (GST_VIDEOSINK_WIDTH (ximagesink) > 0);
+  g_assert (GST_VIDEOSINK_HEIGHT (ximagesink) > 0);
+  if (!ximagesink->xwindow) {
+    ximagesink->xwindow = gst_ximagesink_xwindow_new (ximagesink,
+        GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
+  } else {
+    if (ximagesink->xwindow->internal) {
+      gst_ximagesink_xwindow_resize (ximagesink, ximagesink->xwindow,
+          GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
+    }
+  }
 
-    ximagesink->ximage = gst_ximagesink_ximage_new (ximagesink,
-        GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
-  } else if (!ximagesink->ximage)       /* If no ximage, creating one */
-    ximagesink->ximage = gst_ximagesink_ximage_new (ximagesink,
-        GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
+  /* If our ximage has changed we destroy it, next chain iteration will create
+     a new one */
+  if ((ximagesink->ximage) &&
+      ((GST_VIDEOSINK_WIDTH (ximagesink) != ximagesink->ximage->width) ||
+          (GST_VIDEOSINK_HEIGHT (ximagesink) != ximagesink->ximage->height))) {
+    gst_ximagesink_ximage_destroy (ximagesink, ximagesink->ximage);
+    ximagesink->ximage = NULL;
+  }
 
   gst_x_overlay_got_desired_size (GST_X_OVERLAY (ximagesink),
       GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
@@ -813,6 +1026,10 @@ gst_ximagesink_change_state (GstElement * element)
         ximagesink->xcontext = gst_ximagesink_xcontext_get (ximagesink);
       if (!ximagesink->xcontext)
         return GST_STATE_FAILURE;
+      /* call XSynchronize with the current value of synchronous */
+      GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
+          ximagesink->synchronous ? "TRUE" : "FALSE");
+      XSynchronize (ximagesink->xcontext->disp, ximagesink->synchronous);
       break;
     case GST_STATE_READY_TO_PAUSED:
       if (ximagesink->xwindow)
@@ -877,7 +1094,8 @@ gst_ximagesink_chain (GstPad * pad, GstData * data)
   if (GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
     ximagesink->time = GST_BUFFER_TIMESTAMP (buf);
   }
-  GST_DEBUG ("clock wait: %" GST_TIME_FORMAT, GST_TIME_ARGS (ximagesink->time));
+  GST_LOG_OBJECT (ximagesink, "clock wait: %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (ximagesink->time));
 
   if (GST_VIDEOSINK_CLOCK (ximagesink)) {
     gst_element_wait (GST_ELEMENT (ximagesink), ximagesink->time);
@@ -886,21 +1104,29 @@ gst_ximagesink_chain (GstPad * pad, GstData * data)
   /* If this buffer has been allocated using our buffer management we simply
      put the ximage which is in the PRIVATE pointer */
   if (GST_BUFFER_FREE_DATA_FUNC (buf) == gst_ximagesink_buffer_free) {
+    GST_LOG_OBJECT (ximagesink, "buffer from our pool, writing directly");
     gst_ximagesink_ximage_put (ximagesink, GST_BUFFER_PRIVATE (buf));
-  } else {                      /* Else we have to copy the data into our private image, */
+  } else {
+    /* Else we have to copy the data into our private image, */
     /* if we have one... */
-    if (ximagesink->ximage) {
-      memcpy (ximagesink->ximage->ximage->data,
-          GST_BUFFER_DATA (buf),
-          MIN (GST_BUFFER_SIZE (buf), ximagesink->ximage->size));
-      gst_ximagesink_ximage_put (ximagesink, ximagesink->ximage);
-    } else {                    /* No image available. Something went wrong during capsnego ! */
-
-      gst_buffer_unref (buf);
-      GST_ELEMENT_ERROR (ximagesink, CORE, NEGOTIATION, (NULL),
-          ("no format defined before chain function"));
-      return;
+    GST_LOG_OBJECT (ximagesink, "normal buffer, copying from it");
+    if (!ximagesink->ximage) {
+      GST_DEBUG_OBJECT (ximagesink, "creating our ximage");
+      ximagesink->ximage = gst_ximagesink_ximage_new (ximagesink,
+          GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
+      if (!ximagesink->ximage) {
+        /* No image available. That's very bad ! */
+        gst_buffer_unref (buf);
+        GST_ELEMENT_ERROR (ximagesink, CORE, NEGOTIATION, (NULL),
+            ("Failed creating an XImage in ximagesink chain function."));
+        return;
+      }
     }
+
+    memcpy (ximagesink->ximage->ximage->data,
+        GST_BUFFER_DATA (buf),
+        MIN (GST_BUFFER_SIZE (buf), ximagesink->ximage->size));
+    gst_ximagesink_ximage_put (ximagesink, ximagesink->ximage);
   }
 
   /* set correct time for next buffer */
@@ -930,8 +1156,8 @@ gst_ximagesink_buffer_free (GstBuffer * buffer)
   if ((ximage->width != GST_VIDEOSINK_WIDTH (ximagesink)) ||
       (ximage->height != GST_VIDEOSINK_HEIGHT (ximagesink)))
     gst_ximagesink_ximage_destroy (ximagesink, ximage);
-  else {                        /* In that case we can reuse the image and add it to our image pool. */
-
+  else {
+    /* In that case we can reuse the image and add it to our image pool. */
     g_mutex_lock (ximagesink->pool_lock);
     ximagesink->image_pool = g_slist_prepend (ximagesink->image_pool, ximage);
     g_mutex_unlock (ximagesink->pool_lock);
@@ -960,11 +1186,13 @@ gst_ximagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
       ximagesink->image_pool = g_slist_delete_link (ximagesink->image_pool,
           ximagesink->image_pool);
 
-      if ((ximage->width != GST_VIDEOSINK_WIDTH (ximagesink)) || (ximage->height != GST_VIDEOSINK_HEIGHT (ximagesink))) {       /* This image is unusable. Destroying... */
+      if ((ximage->width != GST_VIDEOSINK_WIDTH (ximagesink)) ||
+          (ximage->height != GST_VIDEOSINK_HEIGHT (ximagesink))) {
+        /* This image is unusable. Destroying... */
         gst_ximagesink_ximage_destroy (ximagesink, ximage);
         ximage = NULL;
-      } else {                  /* We found a suitable image */
-
+      } else {
+        /* We found a suitable image */
         break;
       }
     }
@@ -972,7 +1200,9 @@ gst_ximagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
 
   g_mutex_unlock (ximagesink->pool_lock);
 
-  if (!ximage) {                /* We found no suitable image in the pool. Creating... */
+  if (!ximage) {
+    /* We found no suitable image in the pool. Creating... */
+    GST_DEBUG_OBJECT (ximagesink, "no usable image in pool, creating ximage");
     ximage = gst_ximagesink_ximage_new (ximagesink,
         GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
   }
@@ -1018,7 +1248,7 @@ gst_ximagesink_navigation_send_event (GstNavigation * navigation,
   event = gst_event_new (GST_EVENT_NAVIGATION);
   event->event_data.structure.structure = structure;
 
-  /* We are not converting the pointer coordinates as there's no hardware 
+  /* We are not converting the pointer coordinates as there's no hardware
      scaling done here. The only possible scaling is done by videoscale and
      videoscale will have to catch those events and tranform the coordinates
      to match the applied scaling. So here we just add the offset if the image
@@ -1109,24 +1339,36 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     xwindow->gc = XCreateGC (ximagesink->xcontext->disp, xwindow->win, 0, NULL);
     g_mutex_unlock (ximagesink->x_lock);
 
-    /* If that new window geometry differs from our one we try to 
+    /* If that new window geometry differs from our one we try to
        renegotiate caps */
     if (gst_pad_is_negotiated (GST_VIDEOSINK_PAD (ximagesink)) &&
         (xwindow->width != GST_VIDEOSINK_WIDTH (ximagesink) ||
             xwindow->height != GST_VIDEOSINK_HEIGHT (ximagesink))) {
       GstPadLinkReturn r;
+      GstCaps *caps;
 
-      r = gst_pad_try_set_caps (GST_VIDEOSINK_PAD (ximagesink),
-          gst_caps_new_simple ("video/x-raw-rgb",
-              "bpp", G_TYPE_INT, ximagesink->xcontext->bpp,
-              "depth", G_TYPE_INT, ximagesink->xcontext->depth,
-              "endianness", G_TYPE_INT, ximagesink->xcontext->endianness,
-              "red_mask", G_TYPE_INT, ximagesink->xcontext->visual->red_mask,
-              "green_mask", G_TYPE_INT,
-              ximagesink->xcontext->visual->green_mask, "blue_mask", G_TYPE_INT,
-              ximagesink->xcontext->visual->blue_mask, "width", G_TYPE_INT,
-              xwindow->width, "height", G_TYPE_INT, xwindow->height,
-              "framerate", G_TYPE_DOUBLE, ximagesink->framerate, NULL));
+      caps = gst_caps_new_simple ("video/x-raw-rgb",
+          "bpp", G_TYPE_INT, ximagesink->xcontext->bpp,
+          "depth", G_TYPE_INT, ximagesink->xcontext->depth,
+          "endianness", G_TYPE_INT, ximagesink->xcontext->endianness,
+          "red_mask", G_TYPE_INT, ximagesink->xcontext->visual->red_mask,
+          "green_mask", G_TYPE_INT,
+          ximagesink->xcontext->visual->green_mask,
+          "blue_mask", G_TYPE_INT,
+          ximagesink->xcontext->visual->blue_mask,
+          "width", G_TYPE_INT, xwindow->width,
+          "height", G_TYPE_INT, xwindow->height,
+          "framerate", G_TYPE_DOUBLE, ximagesink->framerate, NULL);
+
+      if (ximagesink->par) {
+        int nom, den;
+
+        nom = gst_value_get_fraction_numerator (ximagesink->par);
+        den = gst_value_get_fraction_denominator (ximagesink->par);
+        gst_caps_set_simple (caps, "pixel-aspect-ratio", GST_TYPE_FRACTION,
+            nom, den, NULL);
+      }
+      r = gst_pad_try_set_caps (GST_VIDEOSINK_PAD (ximagesink), caps);
 
       /* If caps nego succeded updating our size */
       if ((r == GST_PAD_LINK_OK) || (r == GST_PAD_LINK_DONE)) {
@@ -1136,13 +1378,6 @@ gst_ximagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     }
   }
 
-  /* Recreating our ximage */
-  if (!ximagesink->ximage &&
-      GST_VIDEOSINK_WIDTH (ximagesink) && GST_VIDEOSINK_HEIGHT (ximagesink)) {
-    ximagesink->ximage = gst_ximagesink_ximage_new (ximagesink,
-        GST_VIDEOSINK_WIDTH (ximagesink), GST_VIDEOSINK_HEIGHT (ximagesink));
-  }
-
   if (xwindow)
     ximagesink->xwindow = xwindow;
 }
@@ -1209,9 +1444,23 @@ gst_ximagesink_set_property (GObject * object, guint prop_id,
     case ARG_SYNCHRONOUS:
       ximagesink->synchronous = g_value_get_boolean (value);
       if (ximagesink->xcontext) {
+        GST_DEBUG_OBJECT (ximagesink, "XSynchronize called with %s",
+            ximagesink->synchronous ? "TRUE" : "FALSE");
         XSynchronize (ximagesink->xcontext->disp, ximagesink->synchronous);
       }
       break;
+    case ARG_PIXEL_ASPECT_RATIO:
+      g_free (ximagesink->par);
+      ximagesink->par = g_new0 (GValue, 1);
+      g_value_init (ximagesink->par, GST_TYPE_FRACTION);
+      if (!g_value_transform (value, ximagesink->par)) {
+        g_warning ("Could not transform string to aspect ratio");
+        gst_value_set_fraction (ximagesink->par, 1, 1);
+      }
+      GST_DEBUG_OBJECT (ximagesink, "set PAR to %d/%d",
+          gst_value_get_fraction_numerator (ximagesink->par),
+          gst_value_get_fraction_denominator (ximagesink->par));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1235,6 +1484,10 @@ gst_ximagesink_get_property (GObject * object, guint prop_id,
     case ARG_SYNCHRONOUS:
       g_value_set_boolean (value, ximagesink->synchronous);
       break;
+    case ARG_PIXEL_ASPECT_RATIO:
+      if (ximagesink->par)
+        g_value_transform (ximagesink->par, value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1290,7 +1543,6 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
 
   ximagesink->x_lock = g_mutex_new ();
 
-  ximagesink->pixel_width = ximagesink->pixel_height = 1;
 
   ximagesink->image_pool = NULL;
   ximagesink->pool_lock = g_mutex_new ();
@@ -1298,6 +1550,8 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
   ximagesink->sw_scaling_failed = FALSE;
   ximagesink->synchronous = FALSE;
 
+  ximagesink->par = NULL;
+
   GST_FLAG_SET (ximagesink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET (ximagesink, GST_ELEMENT_EVENT_AWARE);
 }
@@ -1331,6 +1585,9 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
       g_param_spec_boolean ("synchronous", "Synchronous", "When enabled, runs "
           "the X display in synchronous mode. (used only for debugging)", FALSE,
           G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, ARG_PIXEL_ASPECT_RATIO,
+      g_param_spec_string ("pixel-aspect-ratio", "Pixel Aspect Ratio",
+          "The pixel aspect ratio of the device", "1/1", G_PARAM_READWRITE));
 
   gobject_class->finalize = gst_ximagesink_finalize;
   gobject_class->set_property = gst_ximagesink_set_property;
